@@ -3,7 +3,7 @@ API for NumPy usage.
 """
 
 import functools
-from typing import Any, Callable, Optional, Sequence, Tuple
+from typing import Any, Callable, List, Optional, Sequence, Tuple
 
 import numpy as np  # type: ignore
 
@@ -24,6 +24,7 @@ from targetran._np_functional import (
     _np_gather_image,
 )
 from targetran._transform import (
+    _AffineDependency,
     _affine_transform,
     _flip_left_right,
     _flip_up_down,
@@ -42,6 +43,16 @@ from targetran._transform import (
 )
 
 
+def _np_get_affine_dependency() -> _AffineDependency:
+    return _AffineDependency(
+        _np_convert, np.shape, np.reshape, np.expand_dims, np.squeeze,
+        _np_pad_image, _np_range, _np_round_to_int, np.repeat, np.tile,
+        np.ones_like, np.stack, np.concatenate, np.matmul, np.clip,
+        _np_gather_image, np.copy, np.max, np.min,
+        _np_logical_and, _np_boolean_mask
+    )
+
+
 def _np_affine_transform(
         image: np.ndarray,
         bboxes: np.ndarray,
@@ -50,13 +61,8 @@ def _np_affine_transform(
         bboxes_tran_mat: np.ndarray
 ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
     return _affine_transform(
-        image, bboxes, labels,
-        _np_convert, np.shape, np.reshape, np.expand_dims, np.squeeze,
-        _np_pad_image, _np_range, _np_round_to_int, np.repeat, np.tile,
-        np.ones_like, np.stack, np.concatenate,
-        image_dest_tran_mat, bboxes_tran_mat, np.matmul, np.clip,
-        _np_gather_image, np.copy, np.max, np.min,
-        _np_logical_and, _np_boolean_mask
+        image, bboxes, labels, image_dest_tran_mat, bboxes_tran_mat,
+        _np_get_affine_dependency()
     )
 
 
@@ -66,12 +72,7 @@ def flip_left_right(
         labels: np.ndarray
 ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
     return _flip_left_right(
-        image, bboxes, labels,
-        _np_convert, np.shape, np.reshape, np.expand_dims, np.squeeze,
-        _np_pad_image, _np_range, _np_round_to_int, np.repeat, np.tile,
-        np.ones_like, np.stack, np.concatenate, np.matmul, np.clip,
-        _np_gather_image, np.copy, np.max, np.min,
-        _np_logical_and, _np_boolean_mask
+        image, bboxes, labels, _np_get_affine_dependency()
     )
 
 
@@ -81,12 +82,7 @@ def flip_up_down(
         labels: np.ndarray
 ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
     return _flip_up_down(
-        image, bboxes, labels,
-        _np_convert, np.shape, np.reshape, np.expand_dims, np.squeeze,
-        _np_pad_image, _np_range, _np_round_to_int, np.repeat, np.tile,
-        np.ones_like, np.stack, np.concatenate, np.matmul, np.clip,
-        _np_gather_image, np.copy, np.max, np.min,
-        _np_logical_and, _np_boolean_mask
+        image, bboxes, labels, _np_get_affine_dependency()
     )
 
 
@@ -97,13 +93,8 @@ def rotate(
         angle_deg: float
 ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
     return _rotate(
-        image, bboxes, labels, _np_convert(angle_deg),
-        _np_convert, np.cos, np.sin, np.shape, np.reshape,
-        np.expand_dims, np.squeeze, _np_pad_image, _np_range,
-        _np_round_to_int, np.repeat, np.tile,
-        np.ones_like, np.stack, np.concatenate, np.matmul,
-        np.clip, _np_gather_image, np.copy, np.max, np.min,
-        _np_logical_and, _np_boolean_mask
+        image, bboxes, labels, _np_convert(angle_deg), np.cos, np.sin,
+        _np_get_affine_dependency()
     )
 
 
@@ -117,12 +108,8 @@ def shear(
     if check_input:
         _check_shear_input(angle_deg)
     return _shear(
-        image, bboxes, labels, _np_convert(angle_deg),
-        _np_convert, np.tan, np.shape, np.reshape, np.expand_dims, np.squeeze,
-        _np_pad_image, _np_range, _np_round_to_int, np.repeat, np.tile,
-        np.ones_like, np.stack, np.concatenate, np.matmul, np.clip,
-        _np_gather_image, np.copy, np.max, np.min,
-        _np_logical_and, _np_boolean_mask
+        image, bboxes, labels, _np_convert(angle_deg), np.tan,
+        _np_get_affine_dependency()
     )
 
 
@@ -139,11 +126,7 @@ def translate(
     return _translate(
         image, bboxes, labels,
         _np_convert(translate_height), _np_convert(translate_width),
-        _np_convert, np.shape, np.reshape, np.expand_dims, np.squeeze,
-        _np_pad_image, _np_range, _np_round_to_int, np.repeat, np.tile,
-        np.ones_like, np.stack, np.concatenate, np.matmul, np.clip,
-        _np_gather_image, np.copy, np.max, np.min,
-        _np_logical_and, _np_boolean_mask
+        _np_get_affine_dependency()
     )
 
 
@@ -166,8 +149,8 @@ def crop(
         labels: np.ndarray,
         offset_height: int,
         offset_width: int,
-        cropped_image_height: int,
-        cropped_image_width: int,
+        crop_height: int,
+        crop_width: int,
         check_input: bool = True
 ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
     if check_input:
@@ -175,7 +158,7 @@ def crop(
     return _crop(
         image, bboxes, labels,
         _np_convert(offset_height), _np_convert(offset_width),
-        _np_convert(cropped_image_height), _np_convert(cropped_image_width),
+        _np_convert(crop_height), _np_convert(crop_width),
         _np_convert, np.shape, np.reshape, np.concatenate,
         _np_logical_and, np.squeeze, np.clip, _np_boolean_mask
     )
@@ -242,7 +225,9 @@ class CombineAffine(RandomTransform):
     def __init__(
             self,
             transforms: Sequence[RandomTransform],
-            probability: float = 0.7,
+            num_selected_transforms: Optional[int] = None,
+            selected_probabilities: Optional[List[float]] = None,
+            probability: float = 1.0,
             seed: Optional[int] = None
     ) -> None:
         not_affine_trans = list(filter(lambda t: not t.is_affine, transforms))
@@ -251,10 +236,18 @@ class CombineAffine(RandomTransform):
                 f"Non-affine transforms cannot be included in CombineAffine: "
                 f"{[t.name for t in not_affine_trans]}"
             )
+        if num_selected_transforms and selected_probabilities:
+            if len(selected_probabilities) != len(transforms):
+                raise ValueError(
+                    "Number of items in selected_probabilities should be "
+                    "the same as the number of items in transforms."
+                )
         super().__init__(
             _np_affine_transform, probability, seed, "CombineAffine", True
         )
         self._transforms = transforms
+        self._num_selected_transforms = num_selected_transforms
+        self._selected_probabilities = selected_probabilities
         self._identity_mat = np.expand_dims(np.array([
             [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]
         ]), axis=0)
@@ -269,13 +262,22 @@ class CombineAffine(RandomTransform):
               for i, t in enumerate(self._transforms)]
         ))
 
-        conditions = np.reshape(rand_fn() < probs, (len(probs), 1, 1))
-        image_dest_tran_mats = np.where(
-            conditions, image_dest_tran_mats, self._identity_mat
-        )
-        bboxes_tran_mats = np.where(
-            conditions, bboxes_tran_mats, self._identity_mat
-        )
+        if self._num_selected_transforms:
+            indices = self._rng.choice(
+                len(self._transforms),
+                self._num_selected_transforms,
+                replace=False, p=self._selected_probabilities
+            ).tolist()
+            image_dest_tran_mats = np.take(image_dest_tran_mats, indices, 0)
+            bboxes_tran_mats = np.take(bboxes_tran_mats, indices, 0)
+        else:
+            conditions = np.reshape(rand_fn() < probs, (len(probs), 1, 1))
+            image_dest_tran_mats = np.where(
+                conditions, image_dest_tran_mats, self._identity_mat
+            )
+            bboxes_tran_mats = np.where(
+                conditions, bboxes_tran_mats, self._identity_mat
+            )
 
         image_dest_tran_mat = functools.reduce(
             np.matmul, image_dest_tran_mats
@@ -306,7 +308,7 @@ class RandomFlipLeftRight(RandomTransform):
 
     def __init__(
             self,
-            probability: float = 0.7,
+            probability: float = 0.5,
             seed: Optional[int] = None
     ) -> None:
         super().__init__(
@@ -335,7 +337,7 @@ class RandomFlipUpDown(RandomTransform):
 
     def __init__(
             self,
-            probability: float = 0.7,
+            probability: float = 0.5,
             seed: Optional[int] = None
     ) -> None:
         super().__init__(
@@ -364,8 +366,8 @@ class RandomRotate(RandomTransform):
 
     def __init__(
             self,
-            angle_deg_range: Tuple[float, float] = (-20.0, 20.0),
-            probability: float = 0.7,
+            angle_deg_range: Tuple[float, float] = (-15.0, 15.0),
+            probability: float = 0.9,
             seed: Optional[int] = None
     ) -> None:
         _check_input_range(angle_deg_range, None, "angle_deg_range")
@@ -402,8 +404,8 @@ class RandomShear(RandomTransform):
 
     def __init__(
             self,
-            angle_deg_range: Tuple[float, float] = (-20.0, 20.0),
-            probability: float = 0.7,
+            angle_deg_range: Tuple[float, float] = (-10.0, 10.0),
+            probability: float = 0.9,
             seed: Optional[int] = None
     ) -> None:
         _check_input_range(angle_deg_range, (-90.0, 90.0), "angle_deg_range")
@@ -438,9 +440,9 @@ class RandomTranslate(RandomTransform):
 
     def __init__(
             self,
-            translate_height_fraction_range: Tuple[float, float] = (-0.2, 0.2),
-            translate_width_fraction_range: Tuple[float, float] = (-0.2, 0.2),
-            probability: float = 0.7,
+            translate_height_fraction_range: Tuple[float, float] = (-0.1, 0.1),
+            translate_width_fraction_range: Tuple[float, float] = (-0.1, 0.1),
+            probability: float = 0.9,
             seed: Optional[int] = None
     ) -> None:
         _check_input_range(
@@ -503,22 +505,20 @@ class RandomCrop(RandomTransform):
 
     def __init__(
             self,
-            cropped_height_fraction_range: Tuple[float, float] = (0.7, 0.9),
-            cropped_width_fraction_range: Tuple[float, float] = (0.7, 0.9),
-            probability: float = 0.7,
+            crop_height_fraction_range: Tuple[float, float] = (0.8, 0.9),
+            crop_width_fraction_range: Tuple[float, float] = (0.8, 0.9),
+            probability: float = 0.9,
             seed: Optional[int] = None
     ) -> None:
         _check_input_range(
-            cropped_height_fraction_range, (0.0, 1.0),
-            "cropped_height_fraction_range"
+            crop_height_fraction_range, (0.0, 1.0), "crop_height_fraction_range"
         )
         _check_input_range(
-            cropped_width_fraction_range, (0.0, 1.0),
-            "cropped_width_fraction_range"
+            crop_width_fraction_range, (0.0, 1.0), "crop_width_fraction_range"
         )
         super().__init__(crop, probability, seed, "RandomCrop", False)
-        self.cropped_height_fraction_range = cropped_height_fraction_range
-        self.cropped_width_fraction_range = cropped_width_fraction_range
+        self.crop_height_fraction_range = crop_height_fraction_range
+        self.crop_width_fraction_range = crop_width_fraction_range
 
     def __call__(
             self,
@@ -528,17 +528,17 @@ class RandomCrop(RandomTransform):
             *args: Any,
             **kwargs: Any
     ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
-        offset_height, offset_width, cropped_height, cropped_width = \
+        offset_height, offset_width, crop_height, crop_width = \
             _np_get_crop_inputs(
                 np.shape(image)[0], np.shape(image)[1],
-                self.cropped_height_fraction_range,
-                self.cropped_width_fraction_range,
+                self.crop_height_fraction_range,
+                self.crop_width_fraction_range,
                 self._rand_fn
             )
 
         return super().__call__(
             image, bboxes, labels,
-            offset_height, offset_width, cropped_height, cropped_width, False
+            offset_height, offset_width, crop_height, crop_width, False
         )
 
 
