@@ -3,7 +3,8 @@ API for TensorFlow usage.
 """
 
 import functools
-from typing import Any, Callable, Optional, Sequence, Tuple, TypeVar
+import itertools
+from typing import Any, Callable, List, Optional, Sequence, Tuple, TypeVar
 
 import numpy as np  # type: ignore
 import tensorflow as tf  # type: ignore
@@ -23,6 +24,7 @@ from targetran._tf_functional import (
     _tf_gather_image,
 )
 from targetran._transform import (
+    _AffineDependency,
     _affine_transform,
     _flip_left_right,
     _flip_up_down,
@@ -55,7 +57,9 @@ def to_tf(
         (_tf_convert(image),
          tf.reshape(_tf_convert(bboxes), (-1, 4)),
          _tf_convert(labels))
-        for image, bboxes, labels in zip(image_seq, bboxes_seq, labels_seq)
+        for image, bboxes, labels in itertools.zip_longest(
+            image_seq, bboxes_seq, labels_seq, fillvalue=[]
+        )
     ]
     tf_image_seq, tf_bboxes_seq, tf_labels_seq = tuple(zip(*tuples))
     return tf_image_seq, tf_bboxes_seq, tf_labels_seq
@@ -82,10 +86,26 @@ def seqs_to_tf_dataset(
     # the whole point of using ragged-tensors is ONLY for building a Dataset...
     # Note that the label ragged-tensors are of rank-0, so they are implicitly
     # converted to tensors during mapping. Strange TF Dataset behaviour...
-    ds = ds.map(lambda i, b, l: (i.to_tensor(), b.to_tensor(), l))
+    ds = ds.map(
+        lambda i, b, l: (
+            i if isinstance(i, tf.Tensor) else i.to_tensor(),
+            b if isinstance(b, tf.Tensor) else b.to_tensor(),
+            l
+        )
+    )
     return ds
 
 
+def _tf_get_affine_dependency() -> _AffineDependency:
+    return _AffineDependency(
+        _tf_convert, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
+        _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
+        tf.ones_like, tf.stack, tf.concat, tf.matmul, tf.clip_by_value,
+        _tf_gather_image, tf.identity, tf.reduce_max, tf.reduce_min,
+        tf.logical_and, tf.boolean_mask
+    )
+
+
 def _tf_affine_transform(
         image: tf.Tensor,
         bboxes: tf.Tensor,
@@ -94,13 +114,8 @@ def _tf_affine_transform(
         bboxes_tran_mat: tf.Tensor
 ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
     return _affine_transform(
-        image, bboxes, labels,
-        _tf_convert, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
-        _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
-        tf.ones_like, tf.stack, tf.concat,
-        image_dest_tran_mat, bboxes_tran_mat, tf.matmul, tf.clip_by_value,
-        _tf_gather_image, tf.identity, tf.reduce_max, tf.reduce_min,
-        tf.logical_and, tf.boolean_mask
+        image, bboxes, labels, image_dest_tran_mat, bboxes_tran_mat,
+        _tf_get_affine_dependency()
     )
 
 
@@ -110,12 +125,7 @@ def tf_flip_left_right(
         labels: tf.Tensor
 ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
     return _flip_left_right(
-        image, bboxes, labels,
-        _tf_convert, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
-        _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
-        tf.ones_like, tf.stack, tf.concat, tf.matmul, tf.clip_by_value,
-        _tf_gather_image, tf.identity, tf.reduce_max, tf.reduce_min,
-        tf.logical_and, tf.boolean_mask
+        image, bboxes, labels, _tf_get_affine_dependency()
     )
 
 
@@ -125,12 +135,7 @@ def tf_flip_up_down(
         labels: tf.Tensor
 ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
     return _flip_up_down(
-        image, bboxes, labels,
-        _tf_convert, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
-        _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
-        tf.ones_like, tf.stack, tf.concat, tf.matmul, tf.clip_by_value,
-        _tf_gather_image, tf.identity, tf.reduce_max, tf.reduce_min,
-        tf.logical_and, tf.boolean_mask
+        image, bboxes, labels, _tf_get_affine_dependency()
     )
 
 
@@ -141,13 +146,8 @@ def tf_rotate(
         angle_deg: float
 ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
     return _rotate(
-        image, bboxes, labels, _tf_convert(angle_deg),
-        _tf_convert, tf.cos, tf.sin, tf.shape, tf.reshape,
-        tf.expand_dims, tf.squeeze, _tf_pad_image, tf.range,
-        _tf_round_to_int, tf.repeat, tf.tile,
-        tf.ones_like, tf.stack, tf.concat, tf.matmul,
-        tf.clip_by_value, _tf_gather_image, tf.identity,
-        tf.reduce_max, tf.reduce_min, tf.logical_and, tf.boolean_mask
+        image, bboxes, labels, _tf_convert(angle_deg), tf.cos, tf.sin,
+        _tf_get_affine_dependency()
     )
 
 
@@ -161,12 +161,8 @@ def tf_shear(
     if check_input:
         _check_shear_input(angle_deg)
     return _shear(
-        image, bboxes, labels, _tf_convert(angle_deg),
-        _tf_convert, tf.tan, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
-        _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
-        tf.ones_like, tf.stack, tf.concat, tf.matmul, tf.clip_by_value,
-        _tf_gather_image, tf.identity, tf.reduce_max, tf.reduce_min,
-        tf.logical_and, tf.boolean_mask
+        image, bboxes, labels, _tf_convert(angle_deg), tf.tan,
+        _tf_get_affine_dependency()
     )
 
 
@@ -185,11 +181,7 @@ def tf_translate(
     return _translate(
         image, bboxes, labels,
         _tf_convert(translate_height), _tf_convert(translate_width),
-        _tf_convert, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
-        _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
-        tf.ones_like, tf.stack, tf.concat, tf.matmul, tf.clip_by_value,
-        _tf_gather_image, tf.identity, tf.reduce_max, tf.reduce_min,
-        tf.logical_and, tf.boolean_mask
+        _tf_get_affine_dependency()
     )
 
 
@@ -212,8 +204,8 @@ def tf_crop(
         labels: tf.Tensor,
         offset_height: int,
         offset_width: int,
-        cropped_image_height: int,
-        cropped_image_width: int,
+        crop_height: int,
+        crop_width: int,
         check_input: bool = True
 ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
     if check_input:
@@ -221,7 +213,7 @@ def tf_crop(
     return _crop(
         image, bboxes, labels,
         _tf_convert(offset_height), _tf_convert(offset_width),
-        _tf_convert(cropped_image_height), _tf_convert(cropped_image_width),
+        _tf_convert(crop_height), _tf_convert(crop_width),
         _tf_convert, tf.shape, tf.reshape, tf.concat,
         tf.logical_and, tf.squeeze, tf.clip_by_value, tf.boolean_mask
     )
@@ -284,12 +276,35 @@ class TFRandomTransform:
         return image, bboxes, labels
 
 
+def _get_random_indices(
+        rng: tf.random.Generator,
+        num_indices: int,
+        num_selected_indices: int,
+        selected_probabilities: Optional[List[float]] = None,
+) -> tf.Tensor:
+    """
+    Roughly mimicking Numpy choice for getting indices, without replacement.
+    The indices always start from 0.
+
+    References:
+    https://github.com/tensorflow/tensorflow/issues/9260#issuecomment-437875125
+    """
+    probs = selected_probabilities if selected_probabilities \
+        else tf.ones(num_indices)
+    logits = tf.math.log(probs)
+    z = -tf.math.log(-tf.math.log(rng.uniform(tf.shape(logits), 0, 1)))
+    _, indices = tf.nn.top_k(logits + z, num_selected_indices)
+    return indices
+
+
 class TFCombineAffine(TFRandomTransform):
 
     def __init__(
             self,
             transforms: Sequence[TFRandomTransform],
-            probability: float = 0.7,
+            num_selected_transforms: Optional[int] = None,
+            selected_probabilities: Optional[List[float]] = None,
+            probability: float = 1.0,
             seed: Optional[int] = None
     ) -> None:
         not_affine_trans = list(filter(lambda t: not t.is_affine, transforms))
@@ -298,10 +313,18 @@ class TFCombineAffine(TFRandomTransform):
                 f"Non-affine transforms cannot be included in TFCombineAffine: "
                 f"{[t.name for t in not_affine_trans]}"
             )
+        if num_selected_transforms and selected_probabilities:
+            if len(selected_probabilities) != len(transforms):
+                raise ValueError(
+                    "Number of items in selected_probabilities should be "
+                    "the same as the number of items in transforms."
+                )
         super().__init__(
             _tf_affine_transform, probability, seed, "TFCombineAffine", True
         )
         self._transforms = transforms
+        self._num_selected_transforms = num_selected_transforms
+        self._selected_probabilities = selected_probabilities
         self._identity_mat = tf.expand_dims(tf.constant([
             [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]
         ]), axis=0)
@@ -316,13 +339,23 @@ class TFCombineAffine(TFRandomTransform):
               for i, t in enumerate(self._transforms)]
         ))
 
-        conditions = tf.reshape(rand_fn() < probs, (len(probs), 1, 1))
-        image_dest_tran_mats = tf.where(
-            conditions, image_dest_tran_mats, self._identity_mat
-        )
-        bboxes_tran_mats = tf.where(
-            conditions, bboxes_tran_mats, self._identity_mat
-        )
+        if self._num_selected_transforms:
+            indices = _get_random_indices(
+                self._rng,
+                len(self._transforms),
+                self._num_selected_transforms,
+                self._selected_probabilities
+            )
+            image_dest_tran_mats = tf.gather(image_dest_tran_mats, indices)
+            bboxes_tran_mats = tf.gather(bboxes_tran_mats, indices)
+        else:
+            conditions = tf.reshape(rand_fn() < probs, (len(probs), 1, 1))
+            image_dest_tran_mats = tf.where(
+                conditions, image_dest_tran_mats, self._identity_mat
+            )
+            bboxes_tran_mats = tf.where(
+                conditions, bboxes_tran_mats, self._identity_mat
+            )
 
         image_dest_tran_mat = functools.reduce(
             tf.matmul, tf.unstack(image_dest_tran_mats)
@@ -353,7 +386,7 @@ class TFRandomFlipLeftRight(TFRandomTransform):
 
     def __init__(
             self,
-            probability: float = 0.7,
+            probability: float = 0.5,
             seed: Optional[int] = None
     ) -> None:
         super().__init__(
@@ -382,7 +415,7 @@ class TFRandomFlipUpDown(TFRandomTransform):
 
     def __init__(
             self,
-            probability: float = 0.7,
+            probability: float = 0.5,
             seed: Optional[int] = None
     ) -> None:
         super().__init__(
@@ -411,8 +444,8 @@ class TFRandomRotate(TFRandomTransform):
 
     def __init__(
             self,
-            angle_deg_range: Tuple[float, float] = (-20.0, 20.0),
-            probability: float = 0.7,
+            angle_deg_range: Tuple[float, float] = (-15.0, 15.0),
+            probability: float = 0.9,
             seed: Optional[int] = None
     ) -> None:
         _check_input_range(angle_deg_range, None, "angle_deg_range")
@@ -449,8 +482,8 @@ class TFRandomShear(TFRandomTransform):
 
     def __init__(
             self,
-            angle_deg_range: Tuple[float, float] = (-20.0, 20.0),
-            probability: float = 0.7,
+            angle_deg_range: Tuple[float, float] = (-10.0, 10.0),
+            probability: float = 0.9,
             seed: Optional[int] = None
     ) -> None:
         _check_input_range(angle_deg_range, (-90.0, 90.0), "angle_deg_range")
@@ -487,9 +520,9 @@ class TFRandomTranslate(TFRandomTransform):
 
     def __init__(
             self,
-            translate_height_fraction_range: Tuple[float, float] = (-0.2, 0.2),
-            translate_width_fraction_range: Tuple[float, float] = (-0.2, 0.2),
-            probability: float = 0.7,
+            translate_height_fraction_range: Tuple[float, float] = (-0.1, 0.1),
+            translate_width_fraction_range: Tuple[float, float] = (-0.1, 0.1),
+            probability: float = 0.9,
             seed: Optional[int] = None
     ) -> None:
         _check_input_range(
@@ -550,22 +583,20 @@ class TFRandomCrop(TFRandomTransform):
 
     def __init__(
             self,
-            cropped_height_fraction_range: Tuple[float, float] = (0.7, 0.9),
-            cropped_width_fraction_range: Tuple[float, float] = (0.7, 0.9),
-            probability: float = 0.7,
+            crop_height_fraction_range: Tuple[float, float] = (0.8, 0.9),
+            crop_width_fraction_range: Tuple[float, float] = (0.8, 0.9),
+            probability: float = 0.9,
             seed: Optional[int] = None
     ) -> None:
         _check_input_range(
-            cropped_height_fraction_range, (0.0, 1.0),
-            "crop_height_fraction_range"
+            crop_height_fraction_range, (0.0, 1.0), "crop_height_fraction_range"
         )
         _check_input_range(
-            cropped_width_fraction_range, (0.0, 1.0),
-            "crop_width_fraction_range"
+            crop_width_fraction_range, (0.0, 1.0), "crop_width_fraction_range"
         )
         super().__init__(tf_crop, probability, seed, "TFRandomCrop", False)
-        self.cropped_height_fraction_range = cropped_height_fraction_range
-        self.cropped_width_fraction_range = cropped_width_fraction_range
+        self.crop_height_fraction_range = crop_height_fraction_range
+        self.crop_width_fraction_range = crop_width_fraction_range
 
     def __call__(
             self,
@@ -575,17 +606,17 @@ class TFRandomCrop(TFRandomTransform):
             *args: Any,
             **kwargs: Any
     ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
-        offset_height, offset_width, cropped_height, cropped_width = \
+        offset_height, offset_width, crop_height, crop_width = \
             _tf_get_crop_inputs(
                 tf.shape(image)[0], tf.shape(image)[1],
-                self.cropped_height_fraction_range,
-                self.cropped_width_fraction_range,
+                self.crop_height_fraction_range,
+                self.crop_width_fraction_range,
                 self._rand_fn
             )
 
         return super().__call__(
             image, bboxes, labels,
-            offset_height, offset_width, cropped_height, cropped_width, False
+            offset_height, offset_width, crop_height, crop_width, False
         )
 
 
